@@ -1,31 +1,28 @@
 use enum_as_inner::EnumAsInner;
 
 #[derive(Debug, EnumAsInner)]
+#[allow(non_camel_case_types)]
+#[allow(clippy::upper_case_acronyms)]
 enum MixedCaseVariants {
     XMLIsNotCool,
-    #[allow(non_camel_case_types)]
     Rust_IsCoolThough(u32),
-    YMCA {
-        named: i16,
-    },
+    YMCA { named: i16 },
 }
 
 #[test]
 fn test_xml_unit() {
     let mixed = MixedCaseVariants::XMLIsNotCool;
 
-    assert!(mixed.as_xml_is_not_cool().is_some());
+    assert!(mixed.is_xml_is_not_cool());
     assert!(mixed.as_rust_is_cool_though().is_none());
     assert!(mixed.as_ymca().is_none());
-
-    assert_eq!(mixed.as_xml_is_not_cool().unwrap(), ());
 }
 
 #[test]
 fn test_rust_unnamed() {
     let mixed = MixedCaseVariants::Rust_IsCoolThough(42);
 
-    assert!(mixed.as_xml_is_not_cool().is_none());
+    assert!(!mixed.is_xml_is_not_cool());
     assert!(mixed.as_rust_is_cool_though().is_some());
     assert!(mixed.as_ymca().is_none());
 
@@ -37,7 +34,7 @@ fn test_rust_unnamed() {
 fn test_ymca_named() {
     let mixed = MixedCaseVariants::YMCA { named: -32_768 };
 
-    assert!(mixed.as_xml_is_not_cool().is_none());
+    assert!(!mixed.is_xml_is_not_cool());
     assert!(mixed.as_rust_is_cool_though().is_none());
     assert!(mixed.as_ymca().is_some());
 
